import argparse
import time
from multiprocessing import Process

from overcooked_simulator.utils import (
    url_and_port_arguments,
    disable_websocket_logging_arguments,
    add_list_of_manager_ids_arguments,
)


def start_game_server(cli_args):
    from overcooked_simulator.game_server import main

    main(cli_args.url, cli_args.port, cli_args.manager_ids)


def start_pygame_gui(cli_args):
    from overcooked_simulator.gui_2d_vis.overcooked_gui import main

    main(cli_args.url, cli_args.port, cli_args.manager_ids)


def main(cli_args=None):
    if cli_args is None:
        parser = argparse.ArgumentParser(
            prog="Overcooked Simulator",
            description="Game Engine Server + PyGameGUI: Starts overcooked game engine server and a PyGame 2D Visualization window in two processes.",
            epilog="For further information, see https://scs.pages.ub.uni-bielefeld.de/cocosy/overcooked-simulator/overcooked_simulator.html",
        )

        url_and_port_arguments(parser)
        disable_websocket_logging_arguments(parser)
        add_list_of_manager_ids_arguments(parser)

        cli_args = parser.parse_args()
    game_server = None
    pygame_gui = None
    try:
        print("Start game engine:")
        game_server = Process(target=start_game_server, args=(cli_args,))
        game_server.start()
        time.sleep(1)
        print("Start PyGame GUI:")
        pygame_gui = Process(target=start_pygame_gui, args=(cli_args,))
        pygame_gui.start()
        while pygame_gui.is_alive() and game_server.is_alive():
            time.sleep(1)
    except KeyboardInterrupt:
        print("Received Keyboard interrupt")
    finally:
        if game_server is not None and game_server.is_alive():
            print("Terminate gparserame server")
            game_server.terminate()
        if pygame_gui is not None and pygame_gui.is_alive():
            print("Terminate pygame gui")
            pygame_gui.terminate()
        time.sleep(0.1)


if __name__ == "__main__":
    main()
